' Play midi notes held in a lookup table
'  	
        Device = 18F452

' Setup the fuses for X4 xtal frequency        
        @CONFIG_REQ
		@__CONFIG CONFIG1H, OSCS_OFF_1 & HSPLL_OSC_1
		@__CONFIG CONFIG2L, BOR_ON_2 & BORV_20_2 & PWRT_ON_2
		@__CONFIG CONFIG2H, WDT_OFF_2 & WDTPS_128_2
		@__CONFIG CONFIG3H, CCP2MX_ON_3
		@__CONFIG CONFIG4L, STVR_ON_4 & LVP_OFF_4 & DEBUG_OFF_4 			     
        
        XTAL = 40
                     
        Dim NOTE_1_COUNTER as Word SYSTEM  		' Determines when the pin for channel 1 is toggled                  
		Dim NOTE_1 as Word SYSTEM 				' Determines the pitch of the note
        Dim NOTE_2_COUNTER as Word SYSTEM    	' Determines when the pin for channel 2 is toggled               
		Dim NOTE_2 as Word SYSTEM 				' Determines the pitch of the note
        Dim NOTE_3_COUNTER as Word SYSTEM   	' Determines when the pin for channel 3 is toggled                
		Dim NOTE_3 as Word SYSTEM 				' Determines the pitch of the note
                        
' Misc variables    
        Dim TICKS as Word
        Dim NOTE_POINTER as Word
        Dim TEMPO as Word
        Dim TUNE_POINTER as Word
        
' Channel variables
		Dim NOTE_STATUS as Byte
        Dim NOTE_1_TO_PLAY as NOTE_STATUS.0
 		Dim NOTE_2_TO_PLAY as NOTE_STATUS.1
        Dim NOTE_3_TO_PLAY as NOTE_STATUS.2
        Dim CHANNEL_1_NOTE as Byte
        Dim CHANNEL_2_NOTE as Byte
		Dim CHANNEL_3_NOTE as Byte
        Dim CHANNEL_ON_TIME as Word
        Dim TRACK_DATA as Word
		
        Symbol TRIGGER_NOTE_1 = TRISB.0
        Symbol TRIGGER_NOTE_2 = TRISB.1
        Symbol TRIGGER_NOTE_3 = TRISB.2
                                                    
        Dim TIMER1 as TMR1L.Word				' Combine TMR1L/TMR1H into 16-bit variable TIMER1
        
		Symbol GIE = INTCON.7					' Global Interrupt Enable/Disable
        
        ON_INTERRUPT Goto NOTE_INT				' Point the hardware interrupt to the interrupt handler
        Delayms 100								' Wait for the PICmicro to stabilise
        ALL_DIGITAL = True						' Set PORTA and PORTE to all digital mode
        Low PORTB								' Discharge the capacitors before we start
        Low PORTA								' Make all of PORTA outputs to help discharge the capacitors
        Delayms 500								' Wait for them to discharge
        Goto MAIN_PROGRAM_LOOP					' Jump over the interrupt handling subroutine and the general subroutines
        
'----[INTERRUPT HANDLER TO PLAY THREE NOTES]--------------------------------------------------
NOTE_INT:
		TIMER1 = 65460							' Load TMR1 to increase the interrupt interval
        			        
        Inc NOTE_1_COUNTER    					' \  
        Inc NOTE_2_COUNTER						'  Increment each channel's toggle counter
        Inc NOTE_3_COUNTER						' /
                     	
' Note generator for channel 1              
        If NOTE_1_COUNTER >= NOTE_1 Then		' Is it time to toggle PORTA.0 ?
        	TRISA = TRISA ^ 1					' Yes. So XOR with 1 to toggle the pin
            Clear NOTE_1_COUNTER				' And reset the toggle counter
        Endif
' Note generator for channel 2
        If NOTE_2_COUNTER >= NOTE_2 Then		' Is it time to toggle PORTA.1 ?
        	TRISA = TRISA ^ 2					' Yes. So XOR with 2 to toggle the pin
            Clear NOTE_2_COUNTER				' And reset the toggle counter
        Endif
' Note generator for channel 3
        If NOTE_3_COUNTER >= NOTE_3 Then		' Is it time to toggle PORTA.2 ?
        	TRISA = TRISA ^ 4					' Yes. So XOR with 3 to toggle the pin
            Clear NOTE_3_COUNTER				' And reset the toggle counter
        Endif       
					
        Clear PIR1.0							' Clear TMR1 interrupt flag
        Retfie FAST								' Return from the interrupt and restore WREG, BSR and STATUS
                 
'-------------------------------------------------------------------------------
' Play a channel's chime based on a midi value
PLAY_NOTE:
		NOTE_STATUS = 255
        If CHANNEL_1_NOTE <> 0 Then								' Play the note held in CHANNEL_1_NOTE ?           	                       	
        	NOTE_POINTER = NOTE_DATA							' Point to the note data table
        	NOTE_POINTER = NOTE_POINTER + (CHANNEL_1_NOTE * 2)	' Locate the correct value for the midi note
        	TRIGGER_NOTE_1 = 0									' Play a chime for this channel
        	NOTE_1 = Lread NOTE_POINTER							' Read the note to play value
        Endif
        Return
              
'----[MAIN PROGRAM CODE]-----------------------------------------------------------        
MAIN_PROGRAM_LOOP:
		     
        NOTE_1_COUNTER = 0						        
		T1CON = %00000001                       ' Turn on Timer1, prescaler = 1:1
    	PIR1.0 = 0                              ' Clear TMR1 interrupt flag
    	PIE1.0 = 1                              ' Enable TMR1 as peripheral interrupt source   	
        INTCON = %11000000                     	' Enable global interrupts, TMR1 will trigger interrupts
        Input PORTB								' Remove any loading to the capacitors
        PORTB = 255								' Set all pins of PORTB high
        
        For CHANNEL_1_NOTE = 24 to 95
        	Gosub PLAY_NOTE
            Delayms 20
            Input PORTB							' Remove the loading from the capacitors to form a chime effect
            Delayms 200
        Next
        Stop									' Stop when all the tunes are played
'-------------------------------------------------------------------------------
        
REMINDERS = OFF 								' Disable any reminders after this point              
REMARKS = OFF									' Disable assembler remarks after this point

' Load the table to convert the midi notes into a frequency value
' for the interrupt sound generator
 
		Include "NOTE_DATA.INC" 
