'
' Produce a tone from PORTB.0 using a TMR1 interrupt
'  	
        Device = 18F452							' Use a PIC18F452 device
        XTAL = 20								' With a 20MHz crystal/resonator
                              
        Dim NOTE_COUNTER as Word SYSTEM 		' Determines when the pin is toggled                  
		Dim NOTE as Word SYSTEM					' Determines the pitch of the note
                      
        Dim TIMER1 as TMR1L.Word				' Combine TMR1L/TMR1H as a 16-bit word
		Symbol GIE  INTCON.7					' Global Interrupt Enable
        
        ON_INTERRUPT Goto NOTE_INT				' Point interrupts to our interrupt handler
        Delayms 400								' Wait for PICmicro to stabilise
        ALL_DIGITAL = True						' Set PORTA and PORTE to digital
        Goto OVER_INTERRUPT						' Jump over the interrupt handler
        
'----[INTERRUPT HANDLER]------------------------------------------------
NOTE_INT:
		TIMER1 = 65490 							' Load TMR1 with a preset value     			        
        Inc NOTE_COUNTER 						' Increment the note counter			
        If NOTE_COUNTER > NOTE Then				' Is it time to toggle the pin ?
        	PORTB = PORTB ^ 1					' Toggle pin PORTB.0
            Clear NOTE_COUNTER					' Clear the note counter
        Endif
		Clear PIR1.0							' Clear TMR1 interrupt flag
        Retfie	FAST							' Exit the interrupt
        
'----[MAIN PROGRAM CODE]--------------------------------------------------       
OVER_INTERRUPT:
		
        Output PORTB.0							' Make PORTB.0 and output
        Clear NOTE_COUNTER						' Reset the note counter
		T1CON = %00000001                       ' Turn on TIMER1, with a 1:1 prescaler
    	Clear PIR1.0                            ' Clear TMR1 interrupt flag
    	Set PIE1.0                              ' Enable TMR1 as peripheral interrupt source   
    	INTCON =  %11000000                     ' Enable global interrupts, peripheral interrupts
        
        NOTE = 0								' Reset the note to play
        Repeat									' Create a loop
        	Inc NOTE							' Lower the pitch of the note
            Delayms 50							' Wait 50ms between notes      	
        Until NOTE = 1000						' Stop when we reach a count of 1000
        Clear GIE								' Disable interrupts to stop the note
        Stop
