' Play a tune consisting of three channels
'
' Program with H4 set and use a 20MHz xtal to overclock the PICmicro to approx 55MHz
'  	
        Device = 18F452

' Setup the fuses for X4 xtal frequency        
        @CONFIG_REQ
		@__CONFIG CONFIG1H, OSCS_OFF_1 & HSPLL_OSC_1
		@__CONFIG CONFIG2L, BOR_ON_2 & BORV_20_2 & PWRT_ON_2
		@__CONFIG CONFIG2H, WDT_OFF_2 & WDTPS_128_2
		@__CONFIG CONFIG3H, CCP2MX_ON_3
		@__CONFIG CONFIG4L, STVR_ON_4 & LVP_OFF_4 & DEBUG_OFF_4 			     
        
        XTAL = 40								' Produce code for 1 40MHz crystal
                     
' Interrupt driven channel variables
        Dim NOTE_1_COUNTER as Word SYSTEM  		' Determines when the pin for channel 1 is toggled                  
		Dim NOTE_1 as Word SYSTEM 				' Determines the pitch of the note
        Dim NOTE_2_COUNTER as Word SYSTEM    	' Determines when the pin for channel 2 is toggled               
		Dim NOTE_2 as Word SYSTEM 				' Determines the pitch of the note
        Dim NOTE_3_COUNTER as Word SYSTEM   	' Determines when the pin for channel 3 is toggled                
		Dim NOTE_3 as Word SYSTEM 				' Determines the pitch of the note
                        
' Misc variables
		Dim NOTE_STATUS as Byte					' Used as Flags
        Dim NOTE_1_TO_PLAY as NOTE_STATUS.0		' 0 = Enable Chime, 1 = No Chime for channel 1
 		Dim NOTE_2_TO_PLAY as NOTE_STATUS.1		' 0 = Enable Chime, 1 = No Chime for channel 2
        Dim NOTE_3_TO_PLAY as NOTE_STATUS.2     ' 0 = Enable Chime, 1 = No Chime for channel 3
        Dim TICKS as Word						' Midi event counter
        Dim NOTE_POINTER as Word				' Pointer for midi to frequency table
        Dim CHANNEL_INFO as Byte				' Used as Flags
      
' Channel 1 variables
 		Dim CHANNEL_1_NOTE as Byte				' Note to play read from tune data for channel 1
        Dim CHANNEL1_TIME_ON as Word			' Midi event trigger for channel 1 chime
        Dim TRACK_1_DATA as Word				' Music data pointer for channel 1
        Dim CHANNEL_1_FINISHED as CHANNEL_INFO.0 ' Indicate if track finished
        
' Channel 2 variables
		Dim CHANNEL_2_NOTE as Byte				' Note to play read from tune data for channel 2
        Dim CHANNEL2_TIME_ON as Word			' Midi event trigger for channel 2 chime
        Dim TRACK_2_DATA as Word				' Music data pointer for channel 2
        Dim CHANNEL_2_FINISHED as CHANNEL_INFO.1 ' Indicate if track finished

' Channel 3 variables
		Dim CHANNEL_3_NOTE as Byte				' Note to play read from tune data for channel 3
        Dim CHANNEL3_TIME_ON as Word			' Midi event trigger for channel 3 chime
        Dim TRACK_3_DATA as Word				' Music data pointer for channel 3
        Dim CHANNEL_3_FINISHED as CHANNEL_INFO.2 ' Indicate if track finished
                               
        Symbol TRIGGER_NOTE_1 = TRISB.0
        Symbol TRIGGER_NOTE_2 = TRISB.1
        Symbol TRIGGER_NOTE_3 = TRISB.2
        Dim TIMER1 as TMR1L.Word				' Combine TMR1L/TMR1H into 16-bit variable TIMER1
        
		Symbol GIE = INTCON.7					' Global Interrupt Enable/Disable
        
        ON_INTERRUPT Goto NOTE_INT				' Point the hardware interrupt to the interrupt handler
        Delayms 100								' Wait for the PICmicro to stabilise
        ALL_DIGITAL = True						' Set PORTA and PORTE to all digital mode
        Low PORTB								' Discharge the capacitors before we start
        Low PORTA								' Make all of PORTA outputs to help discharge the capacitors
        Delayms 500								' Wait for them to discharge
        Goto MAIN_PROGRAM_LOOP					' Jump over the interrupt handling subroutine and the general subroutines
        
'----[INTERRUPT HANDLER TO PLAY THREE NOTES]--------------------------------------------------
NOTE_INT:
		TIMER1 = 65460							' Load TMR1 to increase the interrupt interval
        			        
        Inc NOTE_1_COUNTER    					' \  
        Inc NOTE_2_COUNTER						'  Increment each channel's toggle counter
        Inc NOTE_3_COUNTER						' /
                     	
' Note generator for channel 1              
        If NOTE_1_COUNTER >= NOTE_1 Then		' Is it time to toggle PORTA.0 ?
        	TRISA = TRISA ^ 1					' Yes. So XOR with 1 to toggle the pin
            Clear NOTE_1_COUNTER				' And reset the toggle counter
        Endif
' Note generator for channel 2
        If NOTE_2_COUNTER >= NOTE_2 Then		' Is it time to toggle PORTA.1 ?
        	TRISA = TRISA ^ 2					' Yes. So XOR with 2 to toggle the pin
            Clear NOTE_2_COUNTER				' And reset the toggle counter
        Endif
' Note generator for channel 3
        If NOTE_3_COUNTER >= NOTE_3 Then		' Is it time to toggle PORTA.2 ?
        	TRISA = TRISA ^ 4					' Yes. So XOR with 3 to toggle the pin
            Clear NOTE_3_COUNTER				' And reset the toggle counter
        Endif       
					
        Clear PIR1.0							' Clear TMR1 interrupt flag
        Retfie	FAST							' Return from the interrupt and restore WREG, BSR and STATUS
                 
'------------------------------------------------
' Read the music score data from lookup table TRACK_1
' The data format is time to play the note and the note's midi value
READ_NOTE_DATA_FOR_CHANNEL1:
        CHANNEL1_TIME_ON = Lread TRACK_1_DATA	' Read the NOTE_ON time (16-bits)
        TRACK_1_DATA = TRACK_1_DATA + 2			' Increment to next part
        CHANNEL_1_NOTE = Lread TRACK_1_DATA		' Read the NOTE frequency data (8-bits)       
        TRACK_1_DATA = TRACK_1_DATA + 2			' Increment to next part
        Return 
              
'------------------------------------------------
' Read the music score data from lookup table TRACK_2
' The data format is time to play the note and the note's midi value
READ_NOTE_DATA_FOR_CHANNEL2:
        CHANNEL2_TIME_ON = Lread TRACK_2_DATA	' Read the NOTE_ON time (16-bits)
        TRACK_2_DATA = TRACK_2_DATA + 2			' Increment to next part
        CHANNEL_2_NOTE = Lread TRACK_2_DATA		' Read the NOTE frequency data (8-bits)       
        TRACK_2_DATA = TRACK_2_DATA + 2			' Increment to next part
        Return 
        
'------------------------------------------------
' Read the music score data from lookup table TRACK_3
' The data format is time to play the note and the note's midi value
READ_NOTE_DATA_FOR_CHANNEL3:
        CHANNEL3_TIME_ON = Lread TRACK_3_DATA	' Read the NOTE_ON time (16-bits)
        TRACK_3_DATA = TRACK_3_DATA + 2			' Increment to next part
        CHANNEL_3_NOTE = Lread TRACK_3_DATA		' Read the NOTE frequency data (8-bits)
        TRACK_3_DATA = TRACK_3_DATA + 2			' Increment to next part
        Return 
            
'------------------------------------------------ 
' Take the midi note held in 'NOTE_DATA' previously read from the music score table for TRACK 1
' Convert this into a value for a given frequency using a lookup table
' And play the note on sound channel 1
PLAY_NOTE_1:	
        NOTE_POINTER = NOTE_DATA
        NOTE_POINTER = NOTE_POINTER + (CHANNEL_1_NOTE * 2)
        NOTE_1_TO_PLAY = 0									' Indicate we require a chime for this channel
        NOTE_1 = Lread NOTE_POINTER							' Read the NOTE_1 to play
        Return
        
'------------------------------------------------
' Take the midi note held in 'NOTE_DATA' previously read from the music score table for TRACK 2
' Convert this into a value for a given frequency using a lookup table
' And play the note on sound channel 2   
PLAY_NOTE_2:
		NOTE_POINTER = NOTE_DATA
        NOTE_POINTER = NOTE_POINTER + (CHANNEL_2_NOTE * 2)
        NOTE_2_TO_PLAY = 0									' Indicate we require a chime for this channel
        NOTE_2 = Lread NOTE_POINTER							' Read the NOTE_2 to play
        Return
 
'------------------------------------------------
' Take the midi note held in 'NOTE_DATA' previously read from the music score table for TRACK 3
' Convert this into a value for a given frequency using a lookup table
' And play the note on sound channel 3       
PLAY_NOTE_3:		
        NOTE_POINTER = NOTE_DATA
        NOTE_POINTER = NOTE_POINTER + (CHANNEL_3_NOTE * 2)
        NOTE_3_TO_PLAY = 0									' Indicate we require a chime for this channel
        NOTE_3 = Lread NOTE_POINTER							' Read the NOTE_3 to play
        Return
               
'----[MAIN PROGRAM CODE]-----------------------------------------------------------------------------        
MAIN_PROGRAM_LOOP:
		
        Input PORTB												' Remove any loading to the capacitors
        PORTB = 255												' Set all pins of PORTB high
        NOTE_1_COUNTER = 0										' \
        NOTE_2_COUNTER = 0										'   Clear note counters for all channels
        NOTE_3_COUNTER = 0										' / 
		T1CON = %00000001                         				' Turn on Timer1, prescaler = 1:1
    	PIR1.0 = 0                                				' Clear TMR1 interrupt flag
    	PIE1.0 = 1                                				' Enable TMR1 as peripheral interrupt source   
    	INTCON =  %01000000                       				' Keep global interrupts disabled, TMR1 will trigger interrupts
		
        
        CHANNEL_INFO = 0										' Clear all the channel finished flags
        TRACK_1_DATA = TRACK_1 									' Point TRACK_1_DATA to the music info for TRACK 1
        Gosub READ_NOTE_DATA_FOR_CHANNEL1						' Read the initial times and NOTE_1 data
        TRACK_2_DATA = TRACK_2 									' Point TRACK_2_DATA to the music info for TRACK 2
        Gosub READ_NOTE_DATA_FOR_CHANNEL2						' Read the initial times and note data
        TRACK_3_DATA = TRACK_3 									' Point TRACK_3_DATA to the music info for TRACK 3
        Gosub READ_NOTE_DATA_FOR_CHANNEL3						' Read the initial times and note data
        TICKS = 0												' Reset the ticks counter before we enter the loop
        While 1 = 1												' Create an infinite loop to play the music score                     
            NOTE_STATUS = 255									' Set all notes to OFF by default
            If TICKS = CHANNEL1_TIME_ON  Then					' Have we reached a NOTE_ON time for channel 1?
                If CHANNEL_1_NOTE <> 0 Then 					' Check if there is a note to play               
                	GIE = 1										' Enable the interrupts if channel 1 is to play
                    Gosub PLAY_NOTE_1							' Play the note held in CHANNEL_1_NOTE
                	Gosub READ_NOTE_DATA_FOR_CHANNEL1			' Read the next time and note data                        	
            	Endif
            Else If CHANNEL1_TIME_ON = 0 Then					' Have we reached the end of TRACK 1 ?
            	CHANNEL_1_FINISHED = 1							' Yes. So signal that TRACK 1 is finished
            Endif

            If TICKS = CHANNEL2_TIME_ON  Then					' Have we reached a NOTE_ON time for channel 2?
                If CHANNEL_2_NOTE <> 0 Then 					' Check for channel 2 music data finished               	
                    GIE = 1                						' Enable the interrupts if channel 2 is to play   	
                    Gosub PLAY_NOTE_2							' Play the note held in CHANNEL_2_NOTE
                    Gosub READ_NOTE_DATA_FOR_CHANNEL2			' Read the next time and note data                        	
                Endif
            Else If CHANNEL2_TIME_ON = 0 Then					' Have we reached the end of TRACK 2 ?
            	CHANNEL_2_FINISHED = 1							' Yes. So signal that TRACK 2 is finished
            Endif

            If TICKS = CHANNEL3_TIME_ON  Then					' Have we reached a NOTE_ON time for channel 3?
                If CHANNEL_3_NOTE <> 0 Then 					' Check for channel 3 music data finished              	
                    GIE = 1               						' Enable the interrupts if channel 3 is to play    	                       	
                    Gosub PLAY_NOTE_3							' Play the note held in CHANNEL_3_NOTE
                    Gosub READ_NOTE_DATA_FOR_CHANNEL3			' Read the time and note data                        	
                Endif
            Else If CHANNEL3_TIME_ON = 0 Then					' Have we reached the end of TRACK 3 ?
            	CHANNEL_3_FINISHED = 1							' Yes. So signal that TRACK 3 is finished
            Endif

            If CHANNEL_INFO = %00000111 Then Break				' Exit the loop if all the channels are finished
            
            TRIGGER_NOTE_1 = NOTE_1_TO_PLAY 					' \
        	TRIGGER_NOTE_2 = NOTE_2_TO_PLAY 					'  Enable/Disable the chimes for the three notes
        	TRIGGER_NOTE_3 = NOTE_3_TO_PLAY 					' /
            Delayus 2000										' Delay to allow the capacitors to charge
            Input PORTB											' Turn OFF the notes
            Delayus 4000										' Delay for the tempo of the tune           
            Inc TICKS											' Increment the TICKS counter
        Wend
        Input PORTB												' Turn OFF the notes
        Delayms 3000											' Wait for the notes to decay naturally       
        GIE = 0													' Disable all interrupts
        Stop
'----------------------------------------------------------------
        
REMINDERS = OFF 												' Disable any reminders after this point              
REMARKS = OFF													' Disable assembler remarks after this point
' Lookup table to convert the midi note (0 to 127) into a frequency value
' for the interrupt sound generator
 
		Include "NOTE_DATA.INC"   

'----------------------------------------------------------------
' Load the music score

       	Include "TUNE_DATA.BAS"
        